package compodslimport;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.Usage;
import org.eclipse.uml2.uml.resource.UMLResource;

public class CompoDSLParser 
{
	private ResourceSet resourceSet_;
	private Model model = null;
	private Map<String,org.eclipse.uml2.uml.Type> typemap = new HashMap<String,org.eclipse.uml2.uml.Type>();
	
	public CompoDSLParser(ResourceSet resourceSet)
	{
		resourceSet_ = resourceSet;
		//initTypemap();
	}

	private void initTypemap()
	{
		// Import Primitive Types
		// http://www.eclipse.org/modeling/mdt/uml2/docs/articles/Introduction_to_UML2_Profiles/article.html
		URI pturi = URI.createURI(UMLResource.UML_PRIMITIVE_TYPES_LIBRARY_URI);
		org.eclipse.uml2.uml.Package ptlib = load(pturi);
		typemap.put("UnlimitedNatural",ptlib.getOwnedType("UnlimitedNatural"));
		typemap.put("String",ptlib.getOwnedType("String"));
	}
	
	// aus http://kobra2transformation.googlecode.com/svn-history/r35/trunk/workspace/MM2Profile/src/org/orcas/uml2/UML2Util.java
	public org.eclipse.uml2.uml.Package load(URI uri)
	{
		Resource resource = resourceSet_.getResource(uri, true);

		org.eclipse.uml2.uml.Package package_ = (org.eclipse.uml2.uml.Package) EcoreUtil.getObjectByType(resource.getContents(),
						UMLPackage.Literals.PACKAGE);

		UMLPackage umlPackage = UMLPackage.eINSTANCE;
		resourceSet_.getPackageRegistry().put(umlPackage.getNsURI(), umlPackage);
		resourceSet_.getPackageRegistry().put(package_.getQualifiedName(),package_);
		resourceSet_.getResources().add(umlPackage.eResource());

		return package_;
	}

	public Model getModel()
	{
		return model;
	}

	public Package setModel(Model model)
	{
		this.model = model;
		Package pkg = model;
		return pkg;
	}

	public Package createNewModelAndPackage()
	{
		model = UMLFactory.eINSTANCE.createModel();
		model.setName("Components");
		Package pkg = model;

//		// import elementary types
//		for(String typekey : typemap.keySet())
//		{
//			pkg.createElementImport(typemap.get(typekey));	
//		}
		
		return pkg;
	}

	public Boolean parse(org.eclipse.uml2.uml.Package pkg, IFile inputfile)
			throws CoreException
	{
		String componame = null;
		String filename = inputfile.getName();
		int posend = filename.lastIndexOf('.');
		if (posend > 0)
		{
			componame = filename.substring(0, posend);
		}
		else
		{
			Activator.log(1, "CompoSDLParser: " + filename + " hat kein '.'");
			componame = filename;
		}

		InputStream istr = inputfile.getContents();
		BufferedReader ibff = new BufferedReader(new InputStreamReader(istr));
		String zeile = null;
		
                // http://www.eclipse.org/forums/index.php/m/471902/
		org.eclipse.uml2.uml.Component mycompo = (Component)pkg.createOwnedType(componame, UMLPackage.eINSTANCE.getComponent());
		try
		{
			while ((zeile = ibff.readLine()) != null)
			{
				if (zeile.startsWith("Description"))
				{
					int poscol = zeile.indexOf(':');
					String comment = zeile.substring(poscol + 1);
					Comment ownedComment = mycompo.createOwnedComment();
					ownedComment.setBody(comment);
				}
				else if (zeile.startsWith("port"))
				{
					String[] teile = zeile.split("\\s+");
					if(teile.length < 2)
					{
						Activator.log(1, "Portdescription muss namen haben");
						return false;
					}
					String portname = teile[1];
					String portclassname = portname.substring(0,1).toUpperCase()+portname.substring(1);
					org.eclipse.uml2.uml.Class portclass = (Class)mycompo.getNestedClassifier(portclassname,false,UMLPackage.Literals.CLASS,true);
					Port port = mycompo.getOwnedPort(portname, portclass);
					if(port == null)
					{
					     port = mycompo.createOwnedPort(portname,portclass);
					}
					if(teile.length > 3)
					{
						String interf = teile[3];
						Interface interfo = getOrCreateInterface(interf,pkg);
						if(interfo==null) { continue; }
						if(teile[2].equalsIgnoreCase("require"))
						{
							Usage newusage = UMLFactory.eINSTANCE.createUsage();
							mycompo.getPackagedElements().add(newusage); // usage soll bei Komponente sein
							newusage.getClients().add(portclass);
							newusage.getSuppliers().add(interfo);
						}
						else if(teile[2].equalsIgnoreCase("provide"))
						{	
							InterfaceRealization real = UMLFactory.eINSTANCE.createInterfaceRealization();
							portclass.getInterfaceRealizations().add(real);
							real.setImplementingClassifier(portclass);
							real.setContract(interfo);
						}
						else
						{
							Activator.log(1,"Ungltig: "+teile[2]);
						}
					}
				}
			}
		}
		catch (IOException ioe)
		{
			IStatus status = new Status(IStatus.ERROR, Activator.getId(),
					IStatus.OK, "An error occured during reading '" + filename
							+ "'", ioe);
			throw new CoreException(status);
		}

		// ??? vergleichbares createOwnedDataType fehlt
		// ??? DataType liese sich erzeugen, aber wie kommt er ins Package ?
		// org.eclipse.uml2.uml.DataType myDt =
		// UMLFactory.eINSTANCE.createDataType();
		// myDt.setName(shortname);
		return true;
	}

	org.eclipse.uml2.uml.Interface getOrCreateInterface(String ifname,Package pkg)
	{
		Type res = pkg.getOwnedType(ifname);
		if(res != null)
		{
			org.eclipse.uml2.uml.Interface interf = (org.eclipse.uml2.uml.Interface)res;
			return interf;
		}
		return pkg.createOwnedInterface(ifname);
	}
}
